/*
    Project: TCTEMP
    Version: V1.11
    File   : tc_algidx.c

    Copyright (c) 2006-2008 Author of TCTEMP. All rights reserved.
    Public Key Fingerprint: 75EB 6BC2 01B7 F6E7 4BD7 CC58 4A5F C393 19EE 6E69

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#include "tc.h"

typedef struct {
    unsigned   uIndex;
    const char *paszName;
} CIPHER_NAME, *PCIPHER_NAME;

static CIPHER_NAME cnCiphers[] = {
#ifdef RIJNDAEL
    { TC_AES, "AES" },
    { TC_AES, "AES256" },
    { TC_AES, "Rijndael" },
#endif
#ifdef SERPENT
    { TC_SERPENT, "Serpent" },
#endif
#ifdef TWOFISH
    { TC_TWOFISH, "Twofish" },
#endif
#if defined(RIJNDAEL) && defined(TWOFISH)
    { TC_AES_TWOFISH, "AES-Twofish" },
#endif
#if defined(RIJNDAEL) && defined(TWOFISH) && defined(SERPENT)
    { TC_AES_TWOFISH_SERPENT, "AES-Twofish-Serpent" },
#endif
#if defined(SERPENT) && defined(RIJNDAEL)
    { TC_SERPENT_AES, "Serpent-AES" },
#endif
#if defined(SERPENT) && defined(TWOFISH) && defined(RIJNDAEL)
    { TC_SERPENT_TWOFISH_AES, "Serpent-Twofish-AES" },
#endif
#if defined(TWOFISH) && defined(SERPENT)
    { TC_TWOFISH_SERPENT, "Twofish-Serpent" },
#endif
    { 0, NULL }
};

typedef struct {
    unsigned   uIndex;
    unsigned   uDefaultIterationCount;
    const char *paszName;
} HASH_NAME, *PHASH_NAME;

static HASH_NAME hnHashes[] = {
#ifdef RIPEMD160
    { TC_RIPEMD160, TC_RIPEMD160_ITERATION_COUNT, "RIPEMD-160" },
    { TC_RIPEMD160, TC_RIPEMD160_ITERATION_COUNT, "RIPEMD160" },
#endif
#ifdef SHA512
    { TC_SHA512, TC_SHA512_ITERATION_COUNT, "SHA-512" },
    { TC_SHA512, TC_SHA512_ITERATION_COUNT, "SHA512" },
#endif
#ifdef WHIRLPOOL
    { TC_WHIRLPOOL, TC_WHIRLPOOL_ITERATION_COUNT, "Whirlpool" },
#endif
    { 0, 0, NULL }
};

BOOL __stdcall TC_CipherIsValid(unsigned uCipher)
{
    PCIPHER_NAME p = cnCiphers;

    while (p->uIndex) {
        if (p->uIndex == uCipher) {
            return TRUE;
        }
        p++;
    }

    return FALSE;
}

unsigned __stdcall TC_GetCipherIndex(const char *paszCipherName)
{
    PCIPHER_NAME p = cnCiphers;

    while (p->uIndex) {
        if (stricmp(paszCipherName, p->paszName) == 0) {
            return p->uIndex;
        }
        p++;
    }

    return 0;
}

const char * __stdcall TC_GetCipherName(unsigned uCipher)
{
    PCIPHER_NAME p = cnCiphers;

    while (p->uIndex) {
        if (p->uIndex == uCipher) {
            return p->paszName;
        }
        p++;
    }

    return NULL;
}

BOOL __stdcall TC_HashIsValid(unsigned uHash)
{
    PHASH_NAME p = hnHashes;

    while (p->uIndex) {
        if (p->uIndex == uHash) {
            return TRUE;
        }
        p++;
    }

    return FALSE;
}

unsigned __stdcall TC_GetHashIndex(const char *paszHashName)
{
    PHASH_NAME p = hnHashes;

    while (p->uIndex) {
        if (stricmp(paszHashName, p->paszName) == 0) {
            return p->uIndex;
        }
        p++;
    }

    return 0;
}

const char * __stdcall TC_GetHashName(unsigned uHash)
{
    PHASH_NAME p = hnHashes;

    while (p->uIndex) {
        if (p->uIndex == uHash) {
            return p->paszName;
        }
        p++;
    }

    return NULL;
}

unsigned __stdcall TC_GetDefaultIterationCount(unsigned uHash)
{
    PHASH_NAME p = hnHashes;

    while (p->uIndex) {
        if (p->uIndex == uHash) {
            return p->uDefaultIterationCount;
        }
        p++;
    }

    return 0;
}